/*
   orxonox - the future of 3D-vertical-scrollers

   Copyright (C) 2004 orx

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   ### File Specific:
   main-programmer: Benjamin Grauer
   co-programmer: ...
*/

//#define DEBUG_SPECIAL_MODULE DEBUG_MODULE_

#include "signal_connector.h"

namespace OrxGui
{

  SignalConnector::SignalConnector( )
  {
    this->object = NULL;
    this->exec = NULL;
  }

  SignalConnector::SignalConnector(BaseObject* object, const Executor* executor)
  {
    this->object = object;
    this->exec = executor;
  };

  SignalConnector::SignalConnector(const SignalConnector& signalConnector)
  {
    this->object = signalConnector.object;
    this->exec = (signalConnector.exec == NULL) ? NULL : signalConnector.exec->clone();
  }

  SignalConnector::~SignalConnector()
  {
    delete exec;
  }

  SignalConnector& SignalConnector::operator=(const SignalConnector& signalConnector)
  {
    delete this->exec;
    this->object = signalConnector.object;
    this->exec = (signalConnector.exec == NULL) ? NULL : signalConnector.exec->clone();
  }


  void SignalConnector::operator()() const
  {
    if (this->isValid())
      (*this->exec)(this->object, 0, NULL);
  }

  void SignalConnector::operator()(const MultiType& value0) const
  {
    if (exec != NULL && object != NULL)
      (*this->exec)(this->object, 1, &value0);
  }

  void SignalConnector::operator()(const MultiType& value0, const MultiType& value1) const
  {
    if (exec != NULL && object != NULL)
    {
      MultiType mt[] = { value0, value1 };
      (*this->exec)(this->object, 2, mt);
    }
  }

  void SignalConnector::operator()(const MultiType& value0, const MultiType& value1, const MultiType& value2) const
  {
    if (exec != NULL && object != NULL)
    {
      MultiType mt[] = { value0, value1, value2 };
      (*this->exec)(this->object, 3, mt);
    }
  }

  void SignalConnector::operator()(const MultiType& value0, const MultiType& value1, const MultiType& value2, const MultiType& value3) const
  {
    if (exec != NULL && object != NULL)
    {
      MultiType mt[] = { value0, value1, value2, value3 };
      (*this->exec)(this->object, 4, mt);
    }
  }

  void SignalConnector::operator()(const MultiType& value0, const MultiType& value1, const MultiType& value2, const MultiType& value3, const MultiType& value4) const
  {
    if (exec != NULL && object != NULL)
    {
      MultiType mt[] = { value0, value1, value2, value3, value4 };
      (*this->exec)(this->object, 5, mt);
    }
  }
}
